package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.entidad.Constante;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.ConstanteService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

public class RelatorioCancelamentoTransacao extends Relatorio {

	private ConstanteService constanteService;
	
	public RelatorioCancelamentoTransacao(Map<String, Object> parametros, Connection conexao, ConstanteService constanteService) throws Exception {
		super(parametros, conexao);
		this.constanteService = constanteService;

		this.setCustomDataSource(new ArrayDataSource(this) {

			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();
				NamedParameterStatement stmt = new NamedParameterStatement(conexao, getSql());

				stmt.setTimestamp("dataInicial", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
				stmt.setTimestamp("dataFinal", new Timestamp(DateUtil.fimFecha((Date) parametros.get("DATA_FINAL")).getTime()));

				ResultSet rset = stmt.executeQuery();

				while (rset.next()) {
					Map<String, Object> dataResult = new HashMap<String, Object>();
				
					dataResult.put("empresa", rset.getString("empresa"));
					dataResult.put("dataSolicitacaoCliente", rset.getDate("dataSolicitacaoCliente"));
					dataResult.put("nomePassageiro", rset.getString("nomePassageiro"));
					dataResult.put("dataCompra", rset.getDate("dataCompra"));
					dataResult.put("numeroPassagem", rset.getString("numeroPassagem"));
					dataResult.put("localizador", rset.getString("localizador"));
					dataResult.put("dataViagem", rset.getTimestamp("dataViagem"));
					dataResult.put("servico", rset.getString("servico"));
					dataResult.put("assento", rset.getString("assento"));
					dataResult.put("valorTotalCompra", rset.getBigDecimal("valorTotalCompra"));
					dataResult.put("valorCompra", rset.getBigDecimal("valorCompra"));
					dataResult.put("valorASerRestituido", rset.getBigDecimal("valorASerRestituido"));
					
					this.dados.add(dataResult);
				}

				this.resultSet = rset;
			}
		});
	}

	@Override
	protected void processaParametros() throws Exception {
	}

	private String getSql() {
		StringBuilder sql = new StringBuilder();	
		
		sql.append("WITH OCD_TEMP AS( ");
		sql.append("SELECT ocd.FECINC as dataSolicitacaoCliente, ");
		sql.append("    VALOR_TARJETA as valorASerRestituido, ");
		sql.append("    e.NOMBEMPRESA as empresa, ");
		sql.append("   b.NOMBPASAJERO as nomePassageiro, ");
		sql.append("    c.fechorventa as dataCompra, ");
		sql.append("b.numfoliosistema as numeroPassagem, ");
		sql.append("   b.numoperacion as localizador, ");
		sql.append("    b.FECHORVIAJE as dataViagem, ");
		sql.append("     b.CORRIDA_ID as servico, ");
		sql.append("     b.NUMASIENTO as assento, ");
		sql.append("        c.CAJA_ID as caja_id, ");
		sql.append("  			         b.boletooriginal_id, ");
		sql.append("        			 b.boleto_id, ");
		sql.append("(COALESCE(c.preciopagado,0) + COALESCE(c.importeoutros,0) + COALESCE(c.importepedagio,0) + COALESCE(c.importeseguro,0) + COALESCE(c.importetaxaembarque,0)) AS valorCompra ");
		sql.append("FROM CAJA c ");
		sql.append("INNER JOIN BOLETO b ON c.numoperacion     = b.numoperacion ");
		sql.append("                    AND b.numfoliosistema = c.numfoliosistema ");
		sql.append("INNER JOIN OCD ocd ON ocd.BOLETO_ID       = b.BOLETO_ID ");
		sql.append("INNER JOIN MARCA m ON m.marca_id          = b.marca_id ");
		sql.append("INNER JOIN EMPRESA e ON e.empresa_id      = m.empresa_id ");
		sql.append("WHERE ocd.FECINC                          BETWEEN :dataInicial AND :dataFinal ");
		sql.append("AND ocd.puntoventa_id = ");
		
		Constante constante = constanteService.buscarPorNomeConstante("ID_PUNTO_VENTA_J3_RELATORIO_CANCELAMENTO");
		if(constante != null 
				&& constante.getActivo() 
				&& constante.getValorconstante() != null
				&& !constante.getValorconstante().isEmpty()){
			sql.append(constante.getValorconstante() + " ");
		}else{
			sql.append("99999 ");
		}
		
		if (parametros.get("EMPRESA_ID") != null) {
			sql.append(" AND e.empresa_id = " + parametros.get("EMPRESA_ID"));
		}
		
		sql.append("),");
        sql.append("ocds_total_compra AS ");
        sql.append("(SELECT c.numoperacion   AS numoperacion_cc, ");
        sql.append("(SUM(COALESCE(c.preciopagado,0) ");
        sql.append("+ COALESCE(c.importeoutros,0) ");
        sql.append("+ COALESCE(c.importepedagio,0) ");
        sql.append("+ COALESCE(c.importeseguro,0) ");
        sql.append("+ COALESCE(c.importetaxaembarque,0) ");
        sql.append("+ COALESCE(e.impingreso,0))) ");
        sql.append("+ ");
        sql.append("COALESCE((SELECT ");
        sql.append("SUM(COALESCE(c2.preciopagado,0) ");
        sql.append("+ COALESCE(c2.importeoutros,0) ");
        sql.append("+ COALESCE(c2.importepedagio,0) ");
        sql.append("+ COALESCE(c2.importeseguro,0) ");
        sql.append("+ COALESCE(c2.importetaxaembarque,0) ");
        sql.append("+ COALESCE(e2.impingreso,0)) ");
        sql.append("FROM CAJA c2 ");
        sql.append("INNER JOIN BOLETO b ON c2.numoperacion     = b.numoperacion ");
        sql.append("                     AND b.numfoliosistema = c2.numfoliosistema ");
        sql.append("LEFT JOIN evento_extra e2  ON e2.boleto_id =  b.boleto_id ");
        sql.append("                         AND e2.impingreso > 0 ");
        sql.append("LEFT JOIN  OCD ocd ON ocd.BOLETO_ID        = b.BOLETO_ID ");
        sql.append(" WHERE c2.numoperacion                     = c.numoperacion ");
        sql.append("                            AND ocd.OCD_ID is null),0) ");
        sql.append("AS valorTotalCompra ");
        sql.append("FROM CAJA c ");
        sql.append("INNER JOIN OCD_TEMP o     ON c.caja_id   = o.caja_id ");
        sql.append("LEFT JOIN evento_extra e  ON e.boleto_id = ");
        sql.append("( ");
        sql.append("CASE ");
        sql.append("WHEN o.boletooriginal_id IS NULL ");
        sql.append("THEN o.boleto_id ");
        sql.append("ELSE o.boletooriginal_id ");
        sql.append(" END) ");
        sql.append("AND e.impingreso > 0 ");
        sql.append("GROUP BY c.numoperacion ");
        sql.append(") ");
        
        sql.append("SELECT * FROM OCD_TEMP ocd_temp ");
        sql.append("INNER JOIN ocds_total_compra ocdst ");
        sql.append("ON ocd_temp.localizador = ocdst.numoperacion_cc ");
        
        return sql.toString();
	}

}
