package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.vo.comissao.MovimentosAtrasoVO;
import com.rjconsultores.ventaboletos.vo.comissao.MovimentosAtrasoVO.SituacaoDiaMovimento;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

public class RelatorioMovimentosAtraso extends Relatorio {
	
	private static Logger log = Logger.getLogger(RelatorioMovimentosAtraso.class);
	private Integer empresaId;
	private Integer puntoventaId;
	private Date dataInicial;
	private Date dataFinal;
	private String competencia;
	private SituacaoDiaMovimento status;
	
	public RelatorioMovimentosAtraso(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);
		
		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				definirFiltros(this.relatorio.getParametros());
				List<MovimentosAtrasoVO> lsDadosRelatorio = processarRelatorio(conexao);
				setLsDadosRelatorio(lsDadosRelatorio);
			}
		});
	}

	private List<MovimentosAtrasoVO> processarRelatorio(Connection conexao) throws ParseException {
		List<MovimentosAtrasoVO> lsDadosRelatorio = iniciarDados(conexao);
		ResultSet rset = null;
		NamedParameterStatement stmt = null;
		try {
			StringBuilder sQuery = new StringBuilder();
			sQuery.append("SELECT DISTINCT PV.PUNTOVENTA_ID, CO.DATAMOVIMENTO ")
				  .append("FROM CONFERENCIA CO ")
				  .append("JOIN PUNTO_VENTA PV ON PV.PUNTOVENTA_ID = CO.PUNTOVENTA_ID ")
				  .append("WHERE CO.ACTIVO = 1 ")
				  .append("AND CO.DATAMOVIMENTO BETWEEN :dataInicial AND :dataFinal ")
				  .append("AND CO.INDMALOTERECEBIDO = 1 ");
			
			if(empresaId != null) {
				sQuery.append("AND CO.EMPRESA_ID = :empresaId ");
			}

			if(puntoventaId != null) {
				sQuery.append("AND CO.PUNTOVENTA_ID = :puntoventaId ");
			}

			sQuery.append("ORDER BY PV.PUNTOVENTA_ID, CO.DATAMOVIMENTO");
			
			log.info(sQuery.toString());
			
			stmt = new NamedParameterStatement(conexao, sQuery.toString());
			if(empresaId != null) {
				stmt.setInt("empresaId", empresaId);
			}
			
			if(puntoventaId != null) {
				stmt.setInt("puntoventaId", puntoventaId);
			}
			stmt.setDate("dataInicial", new java.sql.Date(dataInicial.getTime()));
			stmt.setDate("dataFinal", new java.sql.Date(dataFinal.getTime()));
			
			rset = stmt.executeQuery();
			
			while (rset.next()) {
				MovimentosAtrasoVO movimentosAtraso = new MovimentosAtrasoVO(rset.getInt("PUNTOVENTA_ID"),DateUtil.normalizar(rset.getDate("DATAMOVIMENTO")));
				if(lsDadosRelatorio.contains(movimentosAtraso)) {
					movimentosAtraso = lsDadosRelatorio.get(lsDadosRelatorio.indexOf(movimentosAtraso));
					movimentosAtraso.setSituacao(SituacaoDiaMovimento.ENTREGUE);
				}
			}
			
			aplicarFiltroSituacaoMovimentos(lsDadosRelatorio);
			
			Collections.sort(lsDadosRelatorio);
			return lsDadosRelatorio;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			try {
				if(rset != null) {
					rset.close();
				}
				if(stmt != null) {
					stmt.close();
				}
			} catch (SQLException e) {
				log.error(e.getMessage(), e);
			}
		}
		return null;
	}

	private void aplicarFiltroSituacaoMovimentos(List<MovimentosAtrasoVO> lsDadosRelatorio) {
		if(status != null) {
			Set<Integer> sPuntoventaManter = new TreeSet<Integer>();
			for (MovimentosAtrasoVO movimentosAtraso : lsDadosRelatorio) {
				if(movimentosAtraso.getSituacao().equals(status)) {
					sPuntoventaManter.add(movimentosAtraso.getPuntoventaId());
				}
 			}
			
			List<MovimentosAtrasoVO> lsMovimentosAtrasosManter = new ArrayList<MovimentosAtrasoVO>();
			for (MovimentosAtrasoVO movimentosAtraso: lsDadosRelatorio) {
				if(sPuntoventaManter.contains(movimentosAtraso.getPuntoventaId())) {
					lsMovimentosAtrasosManter.add(movimentosAtraso);
				}
			}
			
			if(lsMovimentosAtrasosManter.size() != lsDadosRelatorio.size()) {
				lsDadosRelatorio.clear();
				lsDadosRelatorio.addAll(lsMovimentosAtrasosManter);
			}
		}
	}

	private List<MovimentosAtrasoVO> iniciarDados(Connection conexao) {
		ResultSet rset = null;
		NamedParameterStatement stmt = null;
		try {
			List<MovimentosAtrasoVO> lsMovimentosAtrasoVOs = new ArrayList<MovimentosAtrasoVO>();
			
			StringBuilder sQuery = new StringBuilder();
			sQuery.append("SELECT DISTINCT PV.PUNTOVENTA_ID, PV.NUMPUNTOVENTA, PV.NOMBPUNTOVENTA, NVL(FP.DIASEMTRANSITO,0) AS DIASEMTRANSITO ")
				  .append("FROM PUNTO_VENTA PV ")
				  .append("INNER JOIN PTOVTA_EMPRESA PTE ON PTE.PUNTOVENTA_ID = PV.PUNTOVENTA_ID ")
				  .append("LEFT JOIN FECHAMENTO_PARAMPTOVTA FP ON FP.EMPRESA_ID = PTE.EMPRESA_ID AND PV.PUNTOVENTA_ID = FP.PUNTOVENTA_ID AND FP.ACTIVO = 1 ")
				  .append("WHERE PV.ACTIVO = 1 ");
			
			if(empresaId != null) {
				sQuery.append("AND PTE.EMPRESA_ID = :empresaId ");
			}

			if(puntoventaId != null) {
				sQuery.append("AND PV.PUNTOVENTA_ID = :puntoventaId ");
			}

			sQuery.append("ORDER BY PV.NOMBPUNTOVENTA ");
			
			log.info(sQuery.toString());
			
			stmt = new NamedParameterStatement(conexao, sQuery.toString());
			if(empresaId != null) {
				stmt.setInt("empresaId", empresaId);
			}

			if(puntoventaId != null) {
				stmt.setInt("puntoventaId", puntoventaId);
			}

			rset = stmt.executeQuery();
			while (rset.next()) {
				MovimentosAtrasoVO movimentosAtrasoVO = new MovimentosAtrasoVO();
				movimentosAtrasoVO.setPuntoventaId(rset.getInt("PUNTOVENTA_ID"));
				movimentosAtrasoVO.setNumPuntoVenta(rset.getString("NUMPUNTOVENTA"));
				movimentosAtrasoVO.setNombpuntoventa(rset.getString("NOMBPUNTOVENTA"));
				movimentosAtrasoVO.setCompetencia(competencia);
				movimentosAtrasoVO.setDiasemtransito(rset.getInt("DIASEMTRANSITO"));
				lsMovimentosAtrasoVOs.addAll(carregarDiasMovimento(movimentosAtrasoVO));
			}

			return lsMovimentosAtrasoVOs;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			try {
				if(rset != null) {
					rset.close();
				}
				if(stmt != null) {
					stmt.close();
				}
			} catch (SQLException e) {
				log.error(e.getMessage(), e);
			}
		}
		return null;
	}

	private List<MovimentosAtrasoVO> carregarDiasMovimento(MovimentosAtrasoVO movimentosAtrasoBase) throws ParseException {
		Set<Integer> diasCompetencia = DateUtil.carregarDiasCompetencia(movimentosAtrasoBase.getCompetencia());
		List<MovimentosAtrasoVO> lsMovimentos = new ArrayList<MovimentosAtrasoVO>();
		Date dataAtual = new Date();
		for (Integer dia : diasCompetencia) {
			MovimentosAtrasoVO movimentosAtraso = new MovimentosAtrasoVO(movimentosAtrasoBase.getPuntoventaId(), movimentosAtrasoBase.getNombpuntoventa(),
											  							   movimentosAtrasoBase.getNumPuntoVenta(), movimentosAtrasoBase.getCompetencia(),
											  							   movimentosAtrasoBase.getDiasemtransito(),
											  							   DateUtil.getDateFromString(dia + "/" + movimentosAtrasoBase.getCompetencia(), "dd/MM/yyyy"),
											  							   SituacaoDiaMovimento.AGUARDANDO);
			
			if(movimentosAtraso.getDiasemtransito() != null && movimentosAtraso.getDiasemtransito() > 0) {
				if(DateUtil.getElapsedDias(dataAtual, movimentosAtraso.getDataMovimento()) > movimentosAtraso.getDiasemtransito()) {
					movimentosAtraso.setSituacao(SituacaoDiaMovimento.ATRASADO);
				}
			}
			
			lsMovimentos.add(movimentosAtraso);
		}
		return lsMovimentos;
	}

	public void setLsDadosRelatorio(List<MovimentosAtrasoVO> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
	}

	@Override
	protected void processaParametros() throws Exception {
	}
	
	private void definirFiltros(Map<String, Object> parametros) throws ParseException {
		Integer puntoventaId = (Integer) parametros.get("puntoventaId");
		Integer empresaId = (Integer) parametros.get("empresaId");
		String competencia = (String) parametros.get("competencia");
		status = (SituacaoDiaMovimento) parametros.get("status");
		
		if(StringUtils.isNotBlank(competencia)) {
			this.competencia = competencia;
		}
		
		if(empresaId != null) {
			this.empresaId = empresaId;
		}
		if(puntoventaId != null) {
			this.puntoventaId = puntoventaId;
		}
		
		if(StringUtils.isNotBlank(competencia)) {
			String[] vetCompetencia = competencia.split("/");
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.MONTH, Integer.valueOf(vetCompetencia[0]) - 1);
			cal.set(Calendar.YEAR, Integer.valueOf(vetCompetencia[1]));
	
			this.dataInicial = DateUtil.getDateFromString(cal.getActualMinimum(Calendar.DAY_OF_MONTH) + "/" + competencia, "dd/MM/yyyy");
			this.dataFinal = DateUtil.getDateFromString(cal.getActualMaximum(Calendar.DAY_OF_MONTH) + "/" + competencia, "dd/MM/yyyy");
		}
		
		if(status != null) {
			parametros.put("statusFiltro", status.toString());
		}
		
	}

}
